@@ -9,7 +9,11 @@
 
 import re
 import socket
-import urllib2
+import sys
+try:
+    import urllib2
+except:
+    import urllib.request as urllib2
 
 from Filter import addDetails, printInfo, printWarning
 import Config
@@ -17,9 +21,12 @@ import Config
 # Note: do not add any capturing parentheses here
 macro_regex = re.compile('%+[{(]?[a-zA-Z_]\w{2,}[)}]?')
 
+
 class _HeadRequest(urllib2.Request):
     def get_method(self):
         return "HEAD"
+
+
 class _HeadRedirectHandler(urllib2.HTTPRedirectHandler):
     def redirect_request(*args):
         res = urllib2.HTTPRedirectHandler.redirect_request(*args)
@@ -27,6 +34,7 @@ class _HeadRedirectHandler(urllib2.HTTPRedirectHandler):
             res = _HeadRequest(res.get_full_url())
         return res
 
+
 class AbstractCheck:
     known_checks = {}
 
@@ -39,7 +47,18 @@ class AbstractCheck:
         self.network_timeout = Config.getOption("NetworkTimeout", 10)
 
     def check(self, pkg):
-        raise NotImplementedError('check must be implemented in subclass')
+        if pkg.isSource():
+            return self.check_source(pkg)
+        return self.check_binary(pkg)
+
+    def check_source(self, pkg):
+        return
+
+    def check_binary(self, pkg):
+        return
+
+    def check_spec(self, pkg, spec_file, spec_lines=[]):
+        return
 
     def check_url(self, pkg, tag, url):
         """Check that URL points to something that seems to exist.
@@ -61,7 +80,8 @@ class AbstractCheck:
             opener.addheaders = [('User-Agent',
                                   'rpmlint/%s' % Config.__version__)]
             res = opener.open(_HeadRequest(url))
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             errstr = str(e) or repr(e) or type(e)
             printWarning(pkg, 'invalid-url', '%s:' % tag, url, errstr)
         info = None
@@ -70,19 +90,18 @@ class AbstractCheck:
             res.close()
         return info
 
+
 class AbstractFilesCheck(AbstractCheck):
     def __init__(self, name, file_regexp):
         self.__files_re = re.compile(file_regexp)
         AbstractCheck.__init__(self, name)
-    def check(self, pkg):
-        if pkg.isSource():
-            return
+
+    def check_binary(self, pkg):
         ghosts = pkg.ghostFiles()
         for filename in (x for x in pkg.files() if x not in ghosts):
             if self.__files_re.match(filename):
                 self.check_file(pkg, filename)
 
-
     def check_file(self, pkg, filename):
         """Virtual method called for each file that match the regexp passed
         to the constructor.
