@@ -7,8 +7,10 @@
 # Purpose       : check binary files in a binary rpm package.
 #############################################################################
 
+import os
 import re
 import stat
+import sys
 
 import rpm
 
@@ -22,12 +24,14 @@ DEFAULT_SYSTEM_LIB_PATHS = (
     '/lib', '/usr/lib', '/usr/X11R6/lib',
     '/lib64', '/usr/lib64', '/usr/X11R6/lib64')
 
+
 def create_regexp_call(call):
     if type(call) == type([]):
         call = '(?:' + '|'.join(call) + ')'
     r = "\s+FUNC\s+.*?\s+(%s(?:@GLIBC\S+)?)(?:\s|$)" % call
     return re.compile(r)
 
+
 class BinaryInfo:
 
     needed_regex = re.compile('\s+\(NEEDED\).*\[(\S+)\]')
@@ -43,11 +47,11 @@ class BinaryInfo:
     exit_call_regex = create_regexp_call('_?exit')
     fork_call_regex = create_regexp_call('fork')
     # regexp for setgid setegid setresgid set(?:res|e)?gid
-    setgid_call_regex = create_regexp_call(['setresgid','setegid','setgid'])
-    setuid_call_regex = create_regexp_call(['setresuid','seteuid','setuid'])
-    setgroups_call_regex = create_regexp_call(['initgroups','setgroups'])
+    setgid_call_regex = create_regexp_call(['setresgid', 'setegid', 'setgid'])
+    setuid_call_regex = create_regexp_call(['setresuid', 'seteuid', 'setuid'])
+    setgroups_call_regex = create_regexp_call(['initgroups', 'setgroups'])
     chroot_call_regex = create_regexp_call('chroot')
-    chdir_call_regex  = create_regexp_call('chdir')
+    chdir_call_regex = create_regexp_call('chdir')
     mktemp_call_regex = create_regexp_call('mktemp')
 
     def __init__(self, pkg, path, file, is_ar, is_shlib):
@@ -150,19 +154,23 @@ class BinaryInfo:
             if fork_called:
                 self.exit_calls = []
 
-            # check if chroot is near chdir ( since otherwise, chroot is called without chdir )
+            # check if chroot is near chdir (since otherwise, chroot is called
+            # without chdir)
             if self.chroot and self.chdir:
-                # FIXME this check is too slow, because forking for objdump is quite slow
-                # according to a quick test and that's quite visible on a server like postfix
-                res = Pkg.getstatusoutput(('env', 'LC_ALL=C', 'objdump', '-d', path))
+                # FIXME this check is too slow, because forking for objdump is
+                # quite slow according to a quick test and that's quite visible
+                # on a server like postfix
+                res = Pkg.getstatusoutput(
+                    ('env', 'LC_ALL=C', 'objdump', '-d', path))
                 if not res[0]:
                     call = []
                     # we want that :
                     # 401eb8:   e8 c3 f0 ff ff          callq  400f80 <free@plt>
                     for l in res[1].splitlines():
-                        if l.find('callq ') >= 0:
+                        # call is for x86 32 bits, callq for x86_64
+                        if l.find('callq ') >= 0 or l.find('call ') >= 0:
                             call.append(l.rpartition(' ')[2])
-                    for index,c in enumerate(call):
+                    for index, c in enumerate(call):
                         if c.find('chroot@plt') >= 0:
                             for i in call[index-2:index+2]:
                                 if i.find('chdir@plt'):
@@ -174,10 +182,11 @@ class BinaryInfo:
 
         fobj = None
         try:
-            fobj = open(path)
-            fobj.seek(-12, 2) # 2 == os.SEEK_END, for python 2.4 compat (#172)
-            self.tail = fobj.read()
-        except Exception, e:
+            fobj = open(path, 'rb')
+            fobj.seek(-12, os.SEEK_END)
+            self.tail = Pkg.b2s(fobj.read())
+        except Exception:
+            e = sys.exc_info()[1]
             printWarning(pkg, 'binaryinfo-tail-failed %s: %s' % (file, e))
         if fobj:
             fobj.close()
@@ -243,6 +252,7 @@ srcname_regex = re.compile('(.*?)-[0-9]')
 invalid_dir_ref_regex = re.compile('/(home|tmp)(\W|$)')
 ocaml_mixed_regex = re.compile('^Caml1999X0\d\d$')
 
+
 def dir_base(path):
     res = path_regex.search(path)
     if res:
@@ -250,16 +260,13 @@ def dir_base(path):
     else:
         return '', path
 
+
 class BinariesCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'BinariesCheck')
 
-    def check(self, pkg):
-        # Check only binary package
-        if pkg.isSource():
-            return
-
+    def check_binary(self, pkg):
         files = pkg.files()
         exec_files = []
         has_lib = False
@@ -269,9 +276,11 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
         has_usr_lib_file = False
 
         multi_pkg = False
-        res = srcname_regex.search(pkg[rpm.RPMTAG_SOURCERPM] or '')
-        if res:
-            multi_pkg = (pkg.name != res.group(1))
+        srpm = pkg[rpm.RPMTAG_SOURCERPM]
+        if srpm:
+            res = srcname_regex.search(Pkg.b2s(srpm))
+            if res:
+                multi_pkg = (pkg.name != res.group(1))
 
         for fname, pkgfile in files.items():
 
@@ -359,7 +368,8 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
                                 printError(pkg, 'invalid-ldconfig-symlink',
                                            fname, link)
                         except KeyError:
-                            if base.startswith("lib") or base.startswith("ld-"):
+                            if base.startswith("lib") or \
+                               base.startswith("ld-"):
                                 printError(pkg, 'no-ldconfig-symlink', fname)
 
                     res = soversion_regex.search(bin_info.soname)
@@ -421,7 +431,8 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
                 continue
 
             if not bin_info.needed and not (
-                bin_info.soname and ldso_soname_regex.search(bin_info.soname)):
+                    bin_info.soname and
+                    ldso_soname_regex.search(bin_info.soname)):
                 if is_shobj:
                     printError(pkg,
                                'shared-lib-without-dependency-information',
@@ -432,9 +443,9 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
             else:
                 # linked against libc ?
                 if "libc." not in fname and \
-                        (not bin_info.soname or \
-                             ("libc." not in bin_info.soname and \
-                              not ldso_soname_regex.search(bin_info.soname))):
+                   (not bin_info.soname or
+                    ("libc." not in bin_info.soname and
+                     not ldso_soname_regex.search(bin_info.soname))):
 
                     found_libc = False
                     for lib in bin_info.needed:
@@ -454,12 +465,14 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
                 if bin_info.exec_stack:
                     printWarning(pkg, 'executable-stack', fname)
             elif not bin_info.readelf_error and (
-                pkg.arch.endswith("86") or pkg.arch.startswith("pentium") or
-                pkg.arch in ("athlon", "x86_64")):
+                    pkg.arch.endswith("86") or
+                    pkg.arch.startswith("pentium") or
+                    pkg.arch in ("athlon", "x86_64")):
                 printError(pkg, 'missing-PT_GNU_STACK-section', fname)
 
             if bin_info.setgid and bin_info.setuid and not bin_info.setgroups:
-                printError(pkg, 'missing-call-to-setgroups', fname)
+                printError(pkg, 'missing-call-to-setgroups-before-setuid',
+                           fname)
 
             if bin_info.chroot:
                 if not bin_info.chdir or not bin_info.chroot_near_chdir:
@@ -625,10 +638,10 @@ http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=256900#49''',
 -fPIE/-fpie in compiler flags and -pie in linker flags.''',
 
 'missing-call-to-setgroups-before-setuid',
-'''This executable is calling setuid and setgid without setgroups or initgroups.
-There is a high probability this mean it didn't relinquish all groups, and this
-would be a potential security issue to be fixed. Seek POS36-C on the web for
-details about the problem.''',
+'''This executable is calling setuid and setgid without setgroups or
+initgroups. There is a high probability this means it didn't relinquish all
+groups, and this would be a potential security issue to be fixed. Seek POS36-C
+on the web for details about the problem.''',
 
 'missing-call-to-chdir-with-chroot',
 '''This executable appears to call chroot without using chdir to change the
@@ -645,8 +658,8 @@ upstream to have this issue fixed.''',
 
 'unstripped-binary-or-object',
 '''This executable should be stripped from debugging symbols, in order to take
-less space and be loaded faster. This is usually done automatically at buildtime
-by rpm. Check the build logs and the permission on the file (some
+less space and be loaded faster. This is usually done automatically at
+buildtime by rpm. Check the build logs and the permission on the file (some
 implementations only strip if the permission is 0755).'''
 )
 
