@@ -9,16 +9,15 @@
 #############################################################################
 
 from datetime import datetime
-import commands
 import os
 import re
 import stat
-import string
+import sys
 
 import rpm
 
 from Filter import addDetails, printError, printWarning
-from Pkg import catcmd, is_utf8, is_utf8_str
+from Pkg import b2s, catcmd, getstatusoutput, is_utf8, is_utf8_str
 import AbstractCheck
 import Config
 
@@ -143,14 +142,11 @@ STANDARD_DIRS = (
     '/var/lib/misc',
     '/var/lib/rpm',
     '/var/local',
-    '/var/lock',
-    '/var/lock/subsys',
     '/var/log',
     '/var/mail',
     '/var/nis',
     '/var/opt',
     '/var/preserve',
-    '/var/run',
     '/var/spool',
     '/var/spool/mail',
     '/var/tmp',
@@ -162,14 +158,26 @@ DEFAULT_DANGLING_EXCEPTIONS = (['consolehelper$', 'usermode-consoleonly'],
                                )
 
 # Standard users and groups from LSB Core 4.0.0: 21.2 User & Group Names
-DEFAULT_STANDARD_USERS  = ('root', 'bin', 'daemon', 'adm', 'lp', 'sync',
-                           'shutdown', 'halt', 'mail', 'news', 'uucp',
-                           'operator', 'man', 'nobody',)
+DEFAULT_STANDARD_USERS = ('root', 'bin', 'daemon', 'adm', 'lp', 'sync',
+                          'shutdown', 'halt', 'mail', 'news', 'uucp',
+                          'operator', 'man', 'nobody',)
 DEFAULT_STANDARD_GROUPS = ('root', 'bin', 'daemon', 'adm', 'lp', 'sync',
                            'shutdown', 'halt', 'mail', 'news', 'uucp',
                            'man', 'nobody',)
 
-tmp_regex = re.compile('^(/var|/usr)?/tmp/')
+DEFAULT_DISALLOWED_DIRS = (
+    '/home',
+    '/mnt',
+    '/opt',
+    '/tmp',
+    '/usr/local',
+    '/usr/tmp',
+    '/var/local',
+    '/var/lock',
+    '/var/run',
+    '/var/tmp',
+)
+
 sub_bin_regex = re.compile('^(/usr)?/s?bin/\S+/')
 backup_regex = re.compile('(~|\#[^/]+\#|\.orig|\.rej)$')
 compr_regex = re.compile('\.(gz|z|Z|zip|bz2|lzma|xz)$')
@@ -197,7 +205,7 @@ games_group_regex = re.compile(Config.getOption('RpmGamesGroups', DEFAULT_GAMES_
 dangling_exceptions = Config.getOption('DanglingSymlinkExceptions', DEFAULT_DANGLING_EXCEPTIONS)
 logrotate_regex = re.compile('^/etc/logrotate\.d/(.*)')
 module_rpms_ok = Config.getOption('KernelModuleRPMsOK', True)
-kernel_modules_regex = re.compile('^/lib/modules/(2\.[23456]\.[0-9]+[^/]*?)/')
+kernel_modules_regex = re.compile('^(?:/usr)/lib/modules/([0-9]+\.[0-9]+\.[0-9]+[^/]*?)/')
 kernel_package_regex = re.compile('^kernel(22)?(-)?(smp|enterprise|bigmem|secure|BOOT|i686-up-4GB|p3-smp-64GB)?')
 normal_zero_length_regex = re.compile('^/etc/security/console\.apps/|/\.nosearch$|/__init__\.py$')
 perl_regex = re.compile('^/usr/lib/perl5/(?:vendor_perl/)?([0-9]+\.[0-9]+)\.([0-9]+)/')
@@ -211,17 +219,17 @@ lib_path_regex = re.compile('^(/usr(/X11R6)?)?/lib(64)?')
 lib_package_regex = re.compile('^(lib|.+-libs)')
 hidden_file_regex = re.compile('/\.[^/]*$')
 manifest_perl_regex = re.compile('^/usr/share/doc/perl-.*/MANIFEST(\.SKIP)?$')
-shebang_regex = re.compile('^#!\s*(\S+)')
+shebang_regex = re.compile(b'^#!\s*(\S+)')
 interpreter_regex = re.compile('^/(usr/)?(s?bin|games|libexec(/.+)?|(lib(64)?|share)/.+)/[^/]+$')
 script_regex = re.compile('^/((usr/)?s?bin|etc/(rc\.d/init\.d|X11/xinit\.d|cron\.(hourly|daily|monthly|weekly)))/')
 sourced_script_regex = re.compile('^/etc/(bash_completion\.d|profile\.d)/')
 use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
 skipdocs_regex = re.compile(Config.getOption('SkipDocsRegexp', '\.(?:rtf|x?html?|svg|ml[ily]?)$'), re.IGNORECASE)
 meta_package_regex = re.compile(Config.getOption('MetaPackageRegexp', '^(bundle|task)-'))
-filesys_packages = ['filesystem'] # TODO: make configurable?
+filesys_packages = ['filesystem']  # TODO: make configurable?
 quotes_regex = re.compile('[\'"]+')
 start_certificate_regex = re.compile('^-----BEGIN CERTIFICATE-----$')
-start_private_key_regexp = re.compile('^----BEGIN PRIVATE KEY-----$')
+start_private_key_regex = re.compile('^----BEGIN PRIVATE KEY-----$')
 
 for idx in range(0, len(dangling_exceptions)):
     dangling_exceptions[idx][0] = re.compile(dangling_exceptions[idx][0])
@@ -232,6 +240,8 @@ use_relative_symlinks = Config.getOption("UseRelativeSymlinks", True)
 standard_groups = Config.getOption('StandardGroups', DEFAULT_STANDARD_GROUPS)
 standard_users = Config.getOption('StandardUsers', DEFAULT_STANDARD_USERS)
 
+disallowed_dirs = Config.getOption('DisallowedDirs', DEFAULT_DISALLOWED_DIRS)
+
 non_readable_regexs = (re.compile('^/var/log/'),
                        re.compile('^/etc/(g?shadow-?|securetty)$'))
 
@@ -246,15 +256,22 @@ man_nowarn_regex = re.compile(
     r'(can\'t break|cannot adjust) line')
 man_warn_category = Config.getOption('ManWarningCategory', 'mac')
 
-fsf_license_regex = re.compile('(GNU((\s+(Library|Lesser|Affero))?(\s+General)?\s+Public|\s+Free\s+Documentation)\s+Licen[cs]e|(GP|FD)L)', re.IGNORECASE)
-fsf_wrong_address_regex = re.compile('(675\s+Mass\s+Ave|59\s+Temple\s+Place|Franklin\s+Steet|02139|02111-1307)', re.IGNORECASE)
+fsf_license_regex = re.compile(b'(GNU((\s+(Library|Lesser|Affero))?(\s+General)?\s+Public|\s+Free\s+Documentation)\s+Licen[cs]e|(GP|FD)L)', re.IGNORECASE)
+fsf_wrong_address_regex = re.compile(b'(675\s+Mass\s+Ave|59\s+Temple\s+Place|Franklin\s+Steet|02139|02111-1307)', re.IGNORECASE)
 
 scalable_icon_regex = re.compile(r'^/usr(?:/local)?/share/icons/.*/scalable/')
 
-# loosely inspired from Python Cookbook
-# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/173220
-text_characters = "".join(map(chr, range(32, 127)) + list("\n\r\t\b"))
-_null_trans = string.maketrans("", "")
+# "is binary" stuff borrowed from https://pypi.python.org/pypi/binaryornot
+# TODO: switch to it sometime later instead of embedding our own copy
+
+printable_extended_ascii = b'\n\r\t\f\b'
+if bytes is str:
+    # Python 2 means we need to invoke chr() explicitly
+    printable_extended_ascii += b''.join(map(chr, range(32, 256)))
+else:
+    # Python 3 means bytes accepts integer input directly
+    printable_extended_ascii += bytes(range(32, 256))
+
 
 def peek(filename, pkg, length=1024):
     """Peek into a file, return a chunk from its beginning and a flag if it
@@ -265,13 +282,14 @@ def peek(filename, pkg, length=1024):
         fobj = open(filename, 'rb')
         chunk = fobj.read(length)
         fobj.close()
-    except IOError, e: # eg. https://bugzilla.redhat.com/209876
+    except IOError:  # eg. https://bugzilla.redhat.com/209876
+        e = sys.exc_info()[1]
         printWarning(pkg, 'read-error', e)
         if fobj:
             fobj.close()
         return (chunk, False)
 
-    if "\0" in chunk:
+    if b'\0' in chunk:
         return (chunk, False)
 
     if not chunk:  # Empty files are considered text
@@ -280,18 +298,17 @@ def peek(filename, pkg, length=1024):
     fl = filename.lower()
 
     # PDF's are binary but often detected as text by the algorithm below
-    if fl.endswith('.pdf') and chunk.startswith('%PDF-'):
+    if fl.endswith('.pdf') and chunk.startswith(b'%PDF-'):
         return (chunk, False)
     # Ditto RDoc RI files
     if fl.endswith('.ri') and '/ri/' in fl:
         return (chunk, False)
 
-    # Get the non-text characters (maps a character to itself then
-    # use the 'remove' option to get rid of the text characters.)
-    t = chunk.translate(_null_trans, text_characters)
+    # Binary if control chars are > 30% of the string
+    control_chars = chunk.translate(None, printable_extended_ascii)
+    nontext_ratio = float(len(control_chars)) / float(len(chunk))
+    istext = nontext_ratio <= 0.30
 
-    # If more than 30% non-text characters, then consider it a binary file
-    istext = float(len(t))/len(chunk) <= 0.30
     return (chunk, istext)
 
 # See Python sources for a full list of the values here.
@@ -308,9 +325,10 @@ _python_magic_values = {
     '3.1': 3150,
     '3.2': 3180,
     '3.3': 3230,
-    '3.4': 3260,
+    '3.4': 3310,
     }
 
+
 def get_expected_pyc_magic(path):
     """.pyc/.pyo files embed a 4-byte magic value identifying which version of
     the python bytecode ABI they are for. Given a path to a .pyc/.pyo file,
@@ -342,13 +360,17 @@ def get_expected_pyc_magic(path):
 
     return (expected_magic_value, ver_from_path)
 
+
 def py_demarshal_long(b):
     """Counterpart to Python's PyMarshal_ReadLongFromFile, operating on the
     bytes in a string."""
-    return (ord(b[0])
-            + (ord(b[1]) << 8)
-            + (ord(b[2]) << 16)
-            + (ord(b[3]) << 24))
+    if isinstance(b, str):
+        b = map(ord, b)
+    return (b[0]
+            + (b[1] << 8)
+            + (b[2] << 16)
+            + (b[3] << 24))
+
 
 def python_bytecode_to_script(path):
     """Given a python bytecode path, give the path of the .py file
@@ -364,6 +386,7 @@ def python_bytecode_to_script(path):
 
     return None
 
+
 class FilesCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
@@ -445,19 +468,14 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     is_kernel_package:
                 printError(pkg, "kernel-modules-not-in-kernel-packages", f)
 
-            for i in ['mnt','opt','usr-local','var-local','home']:
-                if f.startswith('/%s/' % i.replace('-','/')):
-                    printError(pkg, 'dir-or-file-in-%s' % i, f)
+            for i in disallowed_dirs:
+                if f.startswith(i):
+                    printError(pkg, 'dir-or-file-in-%s' %
+                               '-'.join(i.split('/')[1:]), f)
 
-            if tmp_regex.search(f):
-                printError(pkg, 'dir-or-file-in-tmp', f)
-
-            elif f.startswith('/var/run/'):
+            if f.startswith('/run/'):
                 if f not in ghost_files:
-                    printWarning(pkg, 'non-ghost-in-var-run', f)
-            elif f.startswith('/var/lock/'):
-                if f not in ghost_files:
-                    printWarning(pkg, 'non-ghost-in-var-lock', f)
+                    printWarning(pkg, 'non-ghost-in-run', f)
             elif sub_bin_regex.search(f):
                 printError(pkg, 'subdir-in-bin', f)
             elif '/site_perl/' in f:
@@ -469,7 +487,8 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                 printError(pkg, 'version-control-internal-file', f)
             elif f.endswith('/.htaccess'):
                 printError(pkg, 'htaccess-file', f)
-            elif hidden_file_regex.search(f) and not f.startswith("/etc/skel/"):
+            elif hidden_file_regex.search(f) and \
+                 not f.startswith("/etc/skel/"):
                 printWarning(pkg, 'hidden-file-or-dir', f)
             elif manifest_perl_regex.search(f):
                 printWarning(pkg, 'manifest-in-perl-module', f)
@@ -489,7 +508,8 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                         printError(pkg, 'compressed-symlink-with-wrong-ext',
                                    f, link)
 
-            perm = mode & 07777
+            perm = mode & int("7777", 8)
+            mode_is_exec = mode & int("111", 8)
 
             if log_regex.search(f):
                 log_file = f
@@ -514,17 +534,17 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                     pkg[rpm.RPMTAG_GROUP]))):
                             printError(pkg, 'setgid-binary', f, group,
                                        oct(perm))
-                    if mode & 0777 != 0755:
+                    if mode & int("777", 8) != int("755", 8):
                         printError(pkg, 'non-standard-executable-perm', f,
                                    oct(perm))
 
                 # Prefetch scriptlets, strip quotes from them (#169)
-                postin = pkg[rpm.RPMTAG_POSTIN] or \
-                    pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
+                postin = b2s(pkg[rpm.RPMTAG_POSTIN]) or \
+                         pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
                 if postin:
                     postin = quotes_regex.sub('', postin)
-                postun = pkg[rpm.RPMTAG_POSTUN] or \
-                    pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
+                postun = b2s(pkg[rpm.RPMTAG_POSTUN]) or \
+                         pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
                 if postun:
                     postun = quotes_regex.sub('', postun)
 
@@ -552,7 +572,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                 if chunk:
                     res = shebang_regex.search(chunk)
                     if res:
-                        interpreter = res.group(1)
+                        interpreter = b2s(res.group(1))
 
                 if doc_regex.search(f):
                     if not interpreter:
@@ -604,8 +624,8 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     elif not install_info_regex.search(postin):
                         printError(pkg, 'postin-without-install-info', f)
 
-                    preun = pkg[rpm.RPMTAG_PREUN] or \
-                        pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
+                    preun = b2s(pkg[rpm.RPMTAG_PREUN]) or \
+                            pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
                     if not postun and not preun:
                         printError(pkg,
                                    'info-files-without-install-info-postun', f)
@@ -628,18 +648,20 @@ class FilesCheck(AbstractCheck.AbstractCheck):
 
                 res = bin_regex.search(f)
                 if res:
-                    if mode & 0111 == 0:
-                        printWarning(pkg, 'non-executable-in-bin', f, oct(perm))
+                    if not mode_is_exec:
+                        printWarning(pkg, 'non-executable-in-bin', f,
+                                     oct(perm))
                     else:
                         exe = res.group(1)
                         if "/" not in exe:
                             bindir_exes.setdefault(exe, []).append(f)
 
                 if not devel_pkg and not is_doc and \
-                       (includefile_regex.search(f) or \
-                        develfile_regex.search(f) or is_buildconfig):
+                   (includefile_regex.search(f) or develfile_regex.search(f) or
+                    is_buildconfig):
                     printWarning(pkg, 'devel-file-in-non-devel-package', f)
-                if mode & 0444 != 0444 and perm & 07000 == 0:
+                if mode & int("444", 8) != int("444", 8) and \
+                   perm & int("7000", 8) == 0:
                     ok_nonreadable = False
                     for regex in non_readable_regexs:
                         if regex.search(f):
@@ -651,7 +673,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                         f not in ghost_files:
                     printError(pkg, 'zero-length', f)
 
-                if mode & 0002 != 0:
+                if mode & stat.S_IWOTH:
                     printError(pkg, 'world-writable', f, oct(perm))
 
                 if not perl_dep_error:
@@ -693,7 +715,8 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                         break
                                 # If expected version was from the file path,
                                 # issue # an error, otherwise a warning.
-                                msg = (pkg, 'python-bytecode-wrong-magic-value',
+                                msg = (pkg,
+                                       'python-bytecode-wrong-magic-value',
                                        f, "expected %d (%s), found %d (%s)" %
                                        (exp_magic,
                                         exp_version or python_default_version,
@@ -716,17 +739,17 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                     pyc_timestamp).isoformat()
                                 sts = datetime.fromtimestamp(
                                     srcfile.mtime).isoformat()
-                                printError(pkg,
-                                           'python-bytecode-inconsistent-mtime',
-                                           f, cts, srcfile.name, sts)
+                                printError(
+                                    pkg, 'python-bytecode-inconsistent-mtime',
+                                    f, cts, srcfile.name, sts)
                     else:
                         printWarning(pkg, 'python-bytecode-without-source', f)
 
                 # normal executable check
-                if mode & stat.S_IXUSR and perm != 0755:
+                if mode & stat.S_IXUSR and perm != int("755", 8):
                     printError(pkg, 'non-standard-executable-perm',
                                f, oct(perm))
-                if mode & 0111 != 0:
+                if mode_is_exec:
                     if f in config_files:
                         printError(pkg, 'executable-marked-as-config-file', f)
                     if not nonexec_file:
@@ -758,11 +781,12 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     man_basenames.add(res.group(1))
                     if use_utf8 and chunk:
                         # TODO: better shell escaping or seq based invocation
-                        cmd = commands.getstatusoutput(
+                        cmd = getstatusoutput(
                             'env LC_ALL=C %s "%s" | gtbl | '
                             'env LC_ALL=en_US.UTF-8 groff -mtty-char -Tutf8 '
                             '-P-c -mandoc -w%s >/dev/null' %
-                            (catcmd(f), pkgfile.path, man_warn_category))
+                            (catcmd(f), pkgfile.path, man_warn_category),
+                            shell=True)
                         for line in cmd[1].split("\n"):
                             res = man_warn_regex.search(line)
                             if not res or man_nowarn_regex.search(line):
@@ -778,7 +802,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                     if f not in ghost_files:
                         if pkg.grep(start_certificate_regex, f):
                             printWarning(pkg, 'pem-certificate', f)
-                        if pkg.grep(start_private_key_regexp, f):
+                        if pkg.grep(start_private_key_regex, f):
                             printError(pkg, 'pem-private-key', f)
 
                 # text file checks
@@ -792,12 +816,11 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                             printError(pkg,
                                        'sourced-script-with-shebang', f,
                                        interpreter)
-                        if mode & 0111 != 0:
+                        if mode_is_exec:
                             printError(pkg, 'executable-sourced-script',
                                        f, oct(perm))
                     # ...but executed ones should
-                    elif interpreter or mode & 0111 != 0 or \
-                            script_regex.search(f):
+                    elif interpreter or mode_is_exec or script_regex.search(f):
                         if interpreter:
                             if not interpreter_regex.search(interpreter):
                                 printError(pkg, 'wrong-script-interpreter',
@@ -807,14 +830,14 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                                  f.endswith('.la')):
                             printError(pkg, 'script-without-shebang', f)
 
-                        if mode & 0111 == 0 and not is_doc:
+                        if not mode_is_exec and not is_doc:
                             printError(pkg, 'non-executable-script', f,
                                        oct(perm), interpreter)
-                        if '\r' in chunk:
+                        if b'\r' in chunk:
                             printError(
                                 pkg, 'wrong-script-end-of-line-encoding', f)
                     elif is_doc and not skipdocs_regex.search(f):
-                        if '\r' in chunk:
+                        if b'\r' in chunk:
                             printWarning(
                                 pkg, 'wrong-file-end-of-line-encoding', f)
                         # We check only doc text files for UTF-8-ness;
@@ -835,16 +858,15 @@ class FilesCheck(AbstractCheck.AbstractCheck):
 
             # normal dir check
             elif stat.S_ISDIR(mode):
-                if mode & 01002 == 2: # world writable without sticky bit
+                if mode & int("1002", 8) == 2:  # world writable w/o sticky bit
                     printError(pkg, 'world-writable', f, oct(perm))
-                if perm != 0755:
+                if perm != int("755", 8):
                     printError(pkg, 'non-standard-dir-perm', f, oct(perm))
                 if pkg.name not in filesys_packages and f in STANDARD_DIRS:
                     printError(pkg, 'standard-dir-owned-by-package', f)
                 if hidden_file_regex.search(f):
                     printWarning(pkg, 'hidden-file-or-dir', f)
 
-
             # symbolic link check
             elif stat.S_ISLNK(mode):
 
@@ -947,7 +969,7 @@ class FilesCheck(AbstractCheck.AbstractCheck):
                 if stat.S_ISLNK(mode):
                     printError(pkg, 'symlink-crontab-file', f)
 
-                if mode & 0111:
+                if mode_is_exec:
                     printError(pkg, 'executable-crontab-file', f)
 
                 if stat.S_IWGRP & mode or stat.S_IWOTH & mode:
@@ -1010,11 +1032,11 @@ a call to ldconfig.''',
 a call to install-info.''',
 
 'postin-without-install-info',
-'''This package contains info files and its %post doesn't call install-info.''',
+'This package contains info files and its %post doesn\'t call install-info.',
 
 'info-files-without-install-info-postun',
-'''This package contains info files and provides no %postun scriptlet containing
-a call to install-info.''',
+'''This package contains info files and provides no %postun scriptlet
+containing a call to install-info.''',
 
 'postun-without-install-info',
 '''This package contains info files and its %postun doesn't call
@@ -1024,35 +1046,10 @@ install-info.''',
 '''You have a perl temporary file in your package. Usually, this
 file is beginning with a dot (.) and contain "perl" in its name.''',
 
-'dir-or-file-in-tmp',
-'''A file in the package is located in /tmp. It's not permitted
-for packages to install files in this directory.''',
-
-'dir-or-file-in-mnt',
-'''A file in the package is located in /mnt. It's not permitted
-for packages to install files in this directory.''',
-
-'dir-or-file-in-opt',
-'''A file in the package is located in /opt. It's not permitted
-for packages to install files in this directory.''',
-
-'dir-or-file-in-usr-local',
-'''A file in the package is located in /usr/local. It's not permitted
-for packages to install files in this directory.''',
-
-'dir-or-file-in-var-local',
-'''A file in the package is located in /var/local. It's not permitted
-for packages to install files in this directory.''',
-
-'non-ghost-in-var-run',
-'''A file or directory in the package is located in /var/run. Files installed
+'non-ghost-in-run',
+'''A file or directory in the package is located in /run. Files installed
 in this directory should be marked as %ghost and created at runtime to work
-properly in tmpfs /var/run setups.''',
-
-'non-ghost-in-var-lock',
-'''A file or directory in the package is located in /var/lock. Files installed
-in this directory should be marked as %ghost and created at runtime to work
-properly in tmpfs /var/lock setups.''',
+properly in tmpfs /run setups.''',
 
 'subdir-in-bin',
 '''The package contains a subdirectory in /usr/bin. It's not permitted to
@@ -1063,10 +1060,6 @@ create a subdir there. Create it in /usr/lib/ instead.''',
 by an editor or resulting from applying unclean (fuzzy, or ones with line
 offsets) patches.''',
 
-'dir-or-file-in-home',
-'''A file in the package is located in /home. It's not permitted
-for packages to install files in this directory.''',
-
 'version-control-internal-file',
 '''You have included file(s) internally used by a version control system
 in the package. Move these files out of the package and rebuild it.''',
@@ -1168,10 +1161,7 @@ target is included in a package in this package's dependency chain.''',
 ''',
 
 'non-readable',
-'''The file can't be read by everybody. If this is expected (for security
-reasons), contact your rpmlint distributor to get it added to the list of
-exceptions for your distro (or add it to your local configuration if you
-installed rpmlint from the source tarball).''',
+'''The file can't be read by everybody. Review if this is expected.''',
 
 'incoherent-logrotate-file',
 '''Your logrotate file should be named /etc/logrotate.d/<package name>.''',
@@ -1366,6 +1356,13 @@ as part of the example documentation mean that someone will sooner or later
 use it and setup a insecure configuration.'''
 )
 
+for i in disallowed_dirs:
+    addDetails('dir-or-file-in-%s' % '-'.join(i.split('/')[1:]),
+    '''A file in the package is located in %s. It's not permitted
+for packages to install files in this directory.''' % i)
+
+
+
 # FilesCheck.py ends here
 
 # Local variables:
