@@ -1,15 +1,16 @@
 # -*- coding: utf-8 -*-
-#---------------------------------------------------------------
+#############################################################################
 # Project         : Mandriva Linux
 # Module          : rpmlint
 # File            : InitScriptCheck.py
 # Author          : Frederic Lepied
 # Created On      : Fri Aug 25 09:26:37 2000
 # Purpose         : check init scripts (files in /etc/rc.d/init.d)
-#---------------------------------------------------------------
+#############################################################################
 
 import os
 import re
+import sys
 
 import rpm
 
@@ -35,16 +36,13 @@ LSB_KEYWORDS = ('Provides', 'Required-Start', 'Required-Stop', 'Should-Start',
 RECOMMENDED_LSB_KEYWORDS = ('Provides', 'Required-Start', 'Required-Stop',
                             'Default-Stop', 'Short-Description')
 
+
 class InitScriptCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'InitScriptCheck')
 
-    def check(self, pkg):
-        # Check only binary package
-        if pkg.isSource():
-            return
-
+    def check_binary(self, pkg):
         initscript_list = []
         for fname, pkgfile in pkg.files().items():
 
@@ -54,22 +52,22 @@ class InitScriptCheck(AbstractCheck.AbstractCheck):
 
             basename = os.path.basename(fname)
             initscript_list.append(basename)
-            if pkgfile.mode & 0500 != 0500:
+            if pkgfile.mode & int("500", 8) != int("500", 8):
                 printError(pkg, 'init-script-non-executable', fname)
 
             if "." in basename:
                 printError(pkg, 'init-script-name-with-dot', fname)
 
             # check chkconfig call in %post and %preun
-            postin = pkg[rpm.RPMTAG_POSTIN] or \
-                pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
+            postin = Pkg.b2s(pkg[rpm.RPMTAG_POSTIN]) or \
+                     pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
             if not postin:
                 printError(pkg, 'init-script-without-chkconfig-postin', fname)
             elif not chkconfig_regex.search(postin):
                 printError(pkg, 'postin-without-chkconfig', fname)
 
-            preun = pkg[rpm.RPMTAG_PREUN] or \
-                pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
+            preun = Pkg.b2s(pkg[rpm.RPMTAG_PREUN]) or \
+                    pkg.scriptprog(rpm.RPMTAG_PREUNPROG)
             if not preun:
                 printError(pkg, 'init-script-without-chkconfig-preun', fname)
             elif not chkconfig_regex.search(preun):
@@ -87,12 +85,13 @@ class InitScriptCheck(AbstractCheck.AbstractCheck):
             content = None
             try:
                 content = Pkg.readlines(pkgfile.path)
-            except Exception, e:
+            except Exception:
+                e = sys.exc_info()[1]
                 printWarning(pkg, 'read-error', e)
                 continue
             content_str = "".join(content)
             for line in content:
-                line = line[:-1] # chomp
+                line = line[:-1]  # chomp
                 # TODO check if there is only one line like this
                 if line.startswith('### BEGIN INIT INFO'):
                     in_lsb_tag = True
@@ -117,9 +116,9 @@ class InitScriptCheck(AbstractCheck.AbstractCheck):
                             cres = lsb_cont_regex.search(line)
                             if not (in_lsb_description and cres):
                                 in_lsb_description = False
-                                printError(pkg,
-                                           'malformed-line-in-lsb-comment-block',
-                                           line)
+                                printError(
+                                    pkg, 'malformed-line-in-lsb-comment-block',
+                                    line)
                             else:
                                 lsb_tags["Description"][-1] += \
                                     " " + cres.group(1)
@@ -157,7 +156,8 @@ class InitScriptCheck(AbstractCheck.AbstractCheck):
                     if use_subsys and name != basename:
                         error = True
                         if name[0] == '$':
-                            value = Pkg.substitute_shell_vars(name, content_str)
+                            value = Pkg.substitute_shell_vars(name,
+                                                              content_str)
                             if value == basename:
                                 error = False
                         else:
