@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
-#---------------------------------------------------------------
+#############################################################################
 # Project         : Mandriva Linux
 # Module          : rpmlint
 # File            : MenuCheck.py
 # Author          : Frederic Lepied
 # Created On      : Mon Mar 20 07:43:37 2000
-#---------------------------------------------------------------
+#############################################################################
 
 import re
 import stat
@@ -126,7 +126,9 @@ DEFAULT_ICON_PATH = (('/usr/share/icons/', 'normal'),
                      ('/usr/share/icons/large/', 'large'))
 
 DEFAULT_LAUNCHERS = (['(?:/usr/bin/)?kdesu', ('/usr/bin/kdesu', 'kdesu')],
-                     ['(?:/usr/bin/)?launch_x11_clanapp', ('/usr/bin/launch_x11_clanapp', 'clanlib', 'libclanlib0')],
+                     ['(?:/usr/bin/)?launch_x11_clanapp',
+                      ('/usr/bin/launch_x11_clanapp', 'clanlib',
+                       'libclanlib0')],
                      ['(?:/usr/bin/)?soundwrapper', None],
                     )
 
@@ -154,16 +156,13 @@ for l in launchers:
     l[0] = re.compile(l[0])
 del l
 
+
 class MenuCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'MenuCheck')
 
-    def check(self, pkg):
-        # Check only binary package
-        if pkg.isSource():
-            return
-
+    def check_binary(self, pkg):
         files = pkg.files()
         menus = []
 
@@ -178,9 +177,9 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                 else:
                     if basename != pkg.name:
                         printWarning(pkg, 'non-coherent-menu-filename', fname)
-                    if mode & 0444 != 0444:
+                    if mode & int("444", 8) != int("444", 8):
                         printError(pkg, 'non-readable-menu-file', fname)
-                    if mode & 0111 != 0:
+                    if mode & int("111", 8):
                         printError(pkg, 'executable-menu-file', fname)
                     menus.append(fname)
             else:
@@ -199,15 +198,15 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                     printError(pkg, 'menu-in-wrong-dir', fname)
 
         if menus:
-            postin = pkg[rpm.RPMTAG_POSTIN] or \
-                pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
+            postin = Pkg.b2s(pkg[rpm.RPMTAG_POSTIN]) or \
+                     pkg.scriptprog(rpm.RPMTAG_POSTINPROG)
             if not postin:
                 printError(pkg, 'menu-without-postin')
             elif not update_menus_regex.search(postin):
                 printError(pkg, 'postin-without-update-menus')
 
-            postun = pkg[rpm.RPMTAG_POSTUN] or \
-                pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
+            postun = Pkg.b2s(pkg[rpm.RPMTAG_POSTUN]) or \
+                     pkg.scriptprog(rpm.RPMTAG_POSTUNPROG)
             if not postun:
                 printError(pkg, 'menu-without-postun')
             elif not update_menus_regex.search(postun):
@@ -260,8 +259,9 @@ class MenuCheck(AbstractCheck.AbstractCheck):
 
                         if command[0] == '/':
                             if command not in files:
-                                printWarning(pkg, 'menu-command-not-in-package',
-                                             command)
+                                printWarning(
+                                    pkg, 'menu-command-not-in-package',
+                                    command)
                         elif not ('/bin/' + command in files or
                                   '/usr/bin/' + command in files or
                                   '/usr/X11R6/bin/' + command in files):
@@ -306,7 +306,7 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                     if res:
                         grp = res.groups()
                         needs = (grp[1] or grp[2]).lower()
-                        if needs in ('x11', 'text' ,'wm'):
+                        if needs in ('x11', 'text', 'wm'):
                             res = section_regex.search(line)
                             if res:
                                 grp = res.groups()
@@ -334,9 +334,9 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                         else:
                             for path in icon_paths:
                                 if (path[0] + icon) not in files:
-                                    printError(pkg,
-                                               path[1] + '-icon-not-in-package',
-                                               icon, f)
+                                    printError(
+                                        pkg, path[1] + '-icon-not-in-package',
+                                        icon, f)
                     else:
                         printWarning(pkg, 'no-icon-in-menu', title)
 
@@ -377,8 +377,8 @@ update-menus.''',
 update-menus.''',
 
 'menu-without-postun',
-'''A menu file exists in the package but no %postun scriptlet is present to call
-update-menus.''',
+'''A menu file exists in the package but no %postun scriptlet is present to
+call update-menus.''',
 
 'postun-without-update-menus',
 '''A menu file exists in the package but its %postun scriptlet doesn't call
