@@ -8,14 +8,16 @@
 #                 the rpm file or by accessing the files contained inside.
 #############################################################################
 
-import commands
 import os
 import re
 import subprocess
 import sys
 import tempfile
-import types
-import urlparse
+import unicodedata
+try:
+    from urlparse import urljoin
+except:
+    from urllib.parse import urljoin
 
 try:
     import magic
@@ -28,13 +30,26 @@ import rpm
 
 import Filter
 
-# Python 2/3 compatibility/convenience wrapper for printing to stderr with
-# less concerns of UnicodeErrors than plain sys.stderr.write.
+# warn(): 2/3 compatibility/convenience wrapper for printing to stderr with
+#         less concerns of UnicodeErrors than plain sys.stderr.write.
+# b2s():  bytes to str
 if sys.version_info[0] > 2:
     # Blows up with Python < 3 without the exec() hack
     exec('def warn(s): print (s, file=sys.stderr)')
+    long = int
+
+    def b2s(b):
+        if b is None:
+            return None
+        if isinstance(b, (list, tuple)):
+            return [b2s(x) for x in b]
+        return b.decode(errors='replace')
 else:
-    def warn(s): print >> sys.stderr, s
+    def warn(s):
+        print >> sys.stderr, s
+
+    def b2s(b):
+        return b
 
 
 # utilities
@@ -49,6 +64,7 @@ PREREQ_FLAG = (rpm.RPMSENSE_PREREQ or 64) | \
 
 var_regex = re.compile('^(.*)\${?(\w+)}?(.*)$')
 
+
 def shell_var_value(var, script):
     assign_regex = re.compile('\\b' + re.escape(var) + '\s*=\s*(.+)\s*(#.*)*$',
                               re.MULTILINE)
@@ -56,12 +72,13 @@ def shell_var_value(var, script):
     if res:
         res2 = var_regex.search(res.group(1))
         if res2:
-            if res2.group(2) == var: # infinite loop
+            if res2.group(2) == var:  # infinite loop
                 return None
         return substitute_shell_vars(res.group(1), script)
     else:
         return None
 
+
 def substitute_shell_vars(val, script):
     res = var_regex.search(val)
     if res:
@@ -73,18 +90,19 @@ def substitute_shell_vars(val, script):
     else:
         return val
 
-def getstatusoutput(cmd, stdoutonly = False):
+
+def getstatusoutput(cmd, stdoutonly=False, shell=False):
     '''A version of commands.getstatusoutput() which can take cmd as a
        sequence, thus making it potentially more secure.'''
     if stdoutonly:
-        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
+        proc = subprocess.Popen(cmd, shell=shell, stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE, close_fds=True)
     else:
-        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
+        proc = subprocess.Popen(cmd, shell=shell, stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT, close_fds=True)
     proc.stdin.close()
-    text = proc.stdout.read()
+    text = b2s(proc.stdout.read())
     sts = proc.wait()
     if sts is None:
         sts = 0
@@ -95,6 +113,7 @@ def getstatusoutput(cmd, stdoutonly = False):
 bz2_regex = re.compile('\.t?bz2?$')
 xz_regex = re.compile('\.(t[xl]z|xz|lzma)$')
 
+
 def catcmd(fname):
     """Get a 'cat' command that handles possibly compressed files."""
     cat = 'gzip -dcf'
@@ -104,17 +123,27 @@ def catcmd(fname):
         cat = 'xz -dc'
     return cat
 
+
 def is_utf8(fname):
     (sts, text) = getstatusoutput(catcmd(fname).split() + [fname])
     return not sts and is_utf8_str(text)
 
-def is_utf8_str(s):
-    try:
-        s.decode('UTF-8')
-    except:
-        return False
-    return True
+REPLACEMENT_CHAR = unicodedata.lookup('REPLACEMENT CHARACTER')
 
+
+def is_utf8_str(s):
+    if hasattr(s, 'decode'):
+        # byte string
+        try:
+            s.decode('UTF-8')
+        except:
+            return False
+        return True
+    # unicode string
+    return REPLACEMENT_CHAR not in s
+
+
+# TODO: PY3
 def to_utf8(string):
     if string is None:
         return ''
@@ -141,15 +170,18 @@ def to_utf8(string):
             newstring = newstring + char
     return newstring
 
+
 def readlines(path):
-    fobj = open(path, "r")
+    fobj = open(path, 'rb')
     try:
-        return fobj.readlines()
+        for line in fobj:
+            yield b2s(line)
     finally:
         fobj.close()
 
+
 def mktemp():
-    tmpfd, tmpname = tempfile.mkstemp(prefix = 'rpmlint.')
+    tmpfd, tmpname = tempfile.mkstemp(prefix='rpmlint.')
     tmpfile = os.fdopen(tmpfd, 'w')
     return tmpfile, tmpname
 
@@ -157,6 +189,7 @@ slash_regex = re.compile('/+')
 slashdot_regex = re.compile('/(\.(/|$))+')
 slashend_regex = re.compile('([^/])/+$')
 
+
 def safe_normpath(path):
     """Like os.path.normpath but normalizes less aggressively thus being
     potentially safer for paths containing symlinks."""
@@ -165,7 +198,8 @@ def safe_normpath(path):
     ret = slashend_regex.sub('\\1', ret)
     return ret
 
-def get_default_valid_rpmgroups(filename = None):
+
+def get_default_valid_rpmgroups(filename=None):
     """Get default rpm groups from filename, or try to look them up from
     the rpm package (if installed) if no filename is given"""
     groups = []
@@ -186,13 +220,16 @@ def get_default_valid_rpmgroups(filename = None):
             fobj.close()
         if 'Development/Debug' not in groups:
             groups.append('Development/Debug')
-        if 'Unspecified' not in groups: # auto-added by rpm >= 4.6.0
+        if 'Unspecified' not in groups:  # auto-added by rpm >= 4.6.0
             groups.append('Unspecified')
         groups.sort()
     return groups
 
+
 # from yum 3.2.27, rpmUtils.miscutils, with rpmlint modifications
-def compareEVR((e1, v1, r1), (e2, v2, r2)):
+def compareEVR(evr1, evr2):
+    (e1, v1, r1) = evr1
+    (e2, v2, r2) = evr2
     # return 1: a is newer than b
     # 0: a and b are the same version
     # -1: b is newer than a
@@ -208,6 +245,7 @@ def compareEVR((e1, v1, r1), (e2, v2, r2)):
     rc = rpm.labelCompare((e1, v1, r1), (e2, v2, r2))
     return rc
 
+
 # from yum 3.2.27, rpmUtils.miscutils, with rpmlint modifications
 def rangeCompare(reqtuple, provtuple):
     """returns true if provtuple satisfies reqtuple"""
@@ -231,7 +269,7 @@ def rangeCompare(reqtuple, provtuple):
     # "Requires: foo < 1.0" should not be satisfied by "Provides: foo = 1:0.5"
     #if reqe is None:
     #    e = None
-    if reqv is None: # just for the record if ver is None then we're going to segfault
+    if reqv is None:  # just for the record if ver is None then we're going to segfault
         v = None
 
     # if we just require foo-version, then foo-version-* will match
@@ -285,6 +323,7 @@ def rangeCompare(reqtuple, provtuple):
 
     return 0
 
+
 # from yum 3.2.23, rpmUtils.miscutils, with rpmlint modifications
 def formatRequire(name, flags, evr):
     s = name
@@ -302,8 +341,9 @@ def formatRequire(name, flags, evr):
             s = "%s %s" % (s, versionToString(evr))
     return s
 
+
 def versionToString(evr):
-    if not isinstance(evr, tuple) and not isinstance(evr, list):
+    if not isinstance(evr, (list, tuple)):
         # assume string
         return evr
     ret = ""
@@ -315,6 +355,7 @@ def versionToString(evr):
             ret += "-" + evr[2]
     return ret
 
+
 # from yum 3.2.23, rpmUtils.miscutils, with some rpmlint modifications
 def stringToVersion(verstring):
     if verstring in (None, ''):
@@ -343,6 +384,7 @@ def stringToVersion(verstring):
         release = None
     return (epoch, version, release)
 
+
 def parse_deps(line):
     '''Parse provides/requires/conflicts/obsoletes line to list of
        (name, flags, (epoch, version, release)) tuples.'''
@@ -412,7 +454,7 @@ class Pkg:
 
     _magic_from_compressed_re = re.compile('\([^)]+\s+compressed\s+data\\b')
 
-    def __init__(self, filename, dirname, header = None, is_source = False):
+    def __init__(self, filename, dirname, header=None, is_source=False):
         self.filename = filename
         self.extracted = False
         self.dirname = dirname
@@ -441,13 +483,13 @@ class Pkg:
                 os.close(fd)
             self.is_source = not self.header[rpm.RPMTAG_SOURCERPM]
 
-        self.name = self.header[rpm.RPMTAG_NAME]
+        self.name = b2s(self.header[rpm.RPMTAG_NAME])
         if self.isNoSource():
             self.arch = 'nosrc'
         elif self.isSource():
             self.arch = 'src'
         else:
-            self.arch = self.header[rpm.RPMTAG_ARCH]
+            self.arch = self.header.format("%{ARCH}")
 
     # Return true if the package is a source package
     def isSource(self):
@@ -467,6 +509,10 @@ class Pkg:
         if val == []:
             return None
         else:
+            if key in (rpm.RPMTAG_VERSION, rpm.RPMTAG_RELEASE, rpm.RPMTAG_ARCH,
+                       rpm.RPMTAG_GROUP, rpm.RPMTAG_BUILDHOST,
+                       rpm.RPMTAG_LICENSE):
+                val = b2s(val)
             return val
 
     # return the name of the directory where the package is extracted
@@ -482,13 +528,13 @@ class Pkg:
             return None
         else:
             self.dirname = tempfile.mkdtemp(
-                prefix = 'rpmlint.%s.' % os.path.basename(self.filename),
-                dir = self.dirname)
+                prefix='rpmlint.%s.' % os.path.basename(self.filename),
+                dir=self.dirname)
             # TODO: better shell escaping or sequence based command invocation
             command_str = \
                 'rpm2cpio "%s" | (cd "%s"; cpio -id); chmod -R +rX "%s"' % \
                 (self.filename, self.dirname, self.dirname)
-            cmd = commands.getstatusoutput(command_str)
+            cmd = getstatusoutput(command_str, shell=True)
             self.extracted = True
             return cmd
 
@@ -507,14 +553,16 @@ class Pkg:
         in_file = None
         try:
             try:
-                in_file = open(self.dirName() + '/' + filename)
+                in_file = open(os.path.join(
+                    self.dirName() or '/', filename.lstrip('/')))
                 for line in in_file:
                     lineno += 1
                     if regex.search(line):
                         ret.append(str(lineno))
                         break
-            except Exception, e:
-                Filter.printWarning(self, 'read-error', filename, e)
+            except Exception:
+                Filter.printWarning(self, 'read-error', filename,
+                                    sys.exc_info()[1])
         finally:
             if in_file:
                 in_file.close()
@@ -525,7 +573,7 @@ class Pkg:
         # LANGUAGE trumps other env vars per GNU gettext docs, see also #166
         orig = os.environ.get('LANGUAGE')
         os.environ['LANGUAGE'] = lang
-        ret = self[tag]
+        ret = b2s(self[tag])
         if orig is not None:
             os.environ['LANGUAGE'] = orig
         return ret
@@ -590,34 +638,32 @@ class Pkg:
         modes = self.header[rpm.RPMTAG_FILEMODES]
         users = self.header[rpm.RPMTAG_FILEUSERNAME]
         groups = self.header[rpm.RPMTAG_FILEGROUPNAME]
-        links = self.header[rpm.RPMTAG_FILELINKTOS]
+        links = [b2s(x) for x in self.header[rpm.RPMTAG_FILELINKTOS]]
         sizes = self.header[rpm.RPMTAG_FILESIZES]
         md5s = self.header[rpm.RPMTAG_FILEMD5S]
         mtimes = self.header[rpm.RPMTAG_FILEMTIMES]
         rdevs = self.header[rpm.RPMTAG_FILERDEVS]
         langs = self.header[rpm.RPMTAG_FILELANGS]
         inodes = self.header[rpm.RPMTAG_FILEINODES]
-        requires = self.header[rpm.RPMTAG_FILEREQUIRE]
-        provides = self.header[rpm.RPMTAG_FILEPROVIDE]
-        files = self.header[rpm.RPMTAG_FILENAMES]
-        magics = self.header[rpm.RPMTAG_FILECLASS]
-        try: # rpm >= 4.7.0
+        requires = [b2s(x) for x in self.header[rpm.RPMTAG_FILEREQUIRE]]
+        provides = [b2s(x) for x in self.header[rpm.RPMTAG_FILEPROVIDE]]
+        files = [b2s(x) for x in self.header[rpm.RPMTAG_FILENAMES]]
+        magics = [b2s(x) for x in self.header[rpm.RPMTAG_FILECLASS]]
+        try:  # rpm >= 4.7.0
             filecaps = self.header[rpm.RPMTAG_FILECAPS]
         except:
             filecaps = None
 
         # rpm-python < 4.6 does not return a list for this (or FILEDEVICES,
         # FWIW) for packages containing exactly one file
-        if not isinstance(inodes, types.ListType):
+        if not isinstance(inodes, list):
             inodes = [inodes]
 
         if files:
             for idx in range(0, len(files)):
                 pkgfile = PkgFile(files[idx])
-                # Do not use os.path.join here, pkgfile.name can start with a
-                # / which would result in self.dirName being ignored
-                pkgfile.path = os.path.normpath(
-                    self.dirName() + '/' + pkgfile.name)
+                pkgfile.path = os.path.normpath(os.path.join(
+                    self.dirName() or '/', pkgfile.name.lstrip('/')))
                 pkgfile.flags = flags[idx]
                 pkgfile.mode = modes[idx]
                 pkgfile.user = users[idx]
@@ -652,7 +698,7 @@ class Pkg:
            PkgFile if it is found in this package, None if not."""
         result = pkgfile
         while result and result.linkto:
-            linkpath = urlparse.urljoin(result.name, result.linkto)
+            linkpath = urljoin(result.name, result.linkto)
             linkpath = safe_normpath(linkpath)
             result = self.files().get(linkpath)
         return result
@@ -706,19 +752,19 @@ class Pkg:
 
     # internal function to gather dependency info used by the above ones
     def _gather_aux(self, header, list, nametag, flagstag, versiontag,
-                    prereq = None):
+                    prereq=None):
         names = header[nametag]
         flags = header[flagstag]
         versions = header[versiontag]
 
         if versions:
             for loop in range(len(versions)):
-                evr = stringToVersion(versions[loop])
+                name = b2s(names[loop])
+                evr = stringToVersion(b2s(versions[loop]))
                 if prereq is not None and flags[loop] & PREREQ_FLAG:
-                    prereq.append((names[loop], flags[loop] & (~PREREQ_FLAG),
-                                   evr))
+                    prereq.append((name, flags[loop] & (~PREREQ_FLAG), evr))
                 else:
-                    list.append((names[loop], flags[loop], evr))
+                    list.append((name, flags[loop], evr))
 
     def _gatherDepInfo(self):
         if self._requires is None:
@@ -752,11 +798,12 @@ class Pkg:
            interpreter arguments, if any."""
         prog = self[which]
         if prog is None:
-            prog = ""
-        elif not isinstance(prog, basestring):
+            prog = b''
+        elif isinstance(prog, (list, tuple)):
             # http://rpm.org/ticket/847#comment:2
-            prog = " ".join(prog)
-        return prog
+            prog = b' '.join(prog)
+        return b2s(prog)
+
 
 def getInstalledPkgs(name):
     """Get list of installed package objects by name."""
@@ -774,9 +821,10 @@ def getInstalledPkgs(name):
 
     return pkgs
 
+
 # Class to provide an API to an installed package
 class InstalledPkg(Pkg):
-    def __init__(self, name, hdr = None):
+    def __init__(self, name, hdr=None):
         if not hdr:
             ts = rpm.TransactionSet()
             mi = ts.dbMatch('name', name)
@@ -801,6 +849,7 @@ class InstalledPkg(Pkg):
     def checkSignature(self):
         return (0, 'fake: pgp md5 OK')
 
+
 # Class to provide an API to a "fake" package, eg. for specfile-only checks
 class FakePkg:
     def __init__(self, name):
@@ -811,6 +860,7 @@ class FakePkg:
     def cleanup(self):
         pass
 
+
 # Class for files in packages
 class PkgFile(object):
 
@@ -836,10 +886,10 @@ class PkgFile(object):
 
     # TODO: decompression support
 
-    is_config    = property(lambda self: self.flags & rpm.RPMFILE_CONFIG)
-    is_doc       = property(lambda self: self.flags & rpm.RPMFILE_DOC)
+    is_config = property(lambda self: self.flags & rpm.RPMFILE_CONFIG)
+    is_doc = property(lambda self: self.flags & rpm.RPMFILE_DOC)
     is_noreplace = property(lambda self: self.flags & rpm.RPMFILE_NOREPLACE)
-    is_ghost     = property(lambda self: self.flags & rpm.RPMFILE_GHOST)
+    is_ghost = property(lambda self: self.flags & rpm.RPMFILE_GHOST)
     is_missingok = property(lambda self: self.flags & rpm.RPMFILE_MISSINGOK)
 
 
