@@ -14,23 +14,19 @@ import AbstractCheck
 import Config
 
 
-DEFAULT_VALID_SRC_PERMS = (0644, 0755)
+DEFAULT_VALID_SRC_PERMS = (int("644", 8), int("755", 8))
 
 source_regex = re.compile('\\.(tar|patch|tgz|diff)$')
 compress_ext = Config.getOption("CompressExtension", "bz2")
 valid_src_perms = Config.getOption("ValidSrcPerms", DEFAULT_VALID_SRC_PERMS)
 
-class SourceCheck(AbstractCheck.AbstractCheck):
 
+class SourceCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'SourceCheck')
 
-    def check(self, pkg):
-        # Check only source package
-        if not pkg.isSource():
-            return
-
+    def check_source(self, pkg):
         # process file list
         spec_file = None
         for fname, pkgfile in pkg.files().items():
@@ -43,7 +39,7 @@ class SourceCheck(AbstractCheck.AbstractCheck):
                     not fname.endswith(compress_ext):
                 printWarning(pkg, 'source-or-patch-not-compressed',
                              compress_ext, fname)
-            perm = pkgfile.mode & 07777
+            perm = pkgfile.mode & int("7777", 8)
             if perm not in valid_src_perms:
                 printWarning(pkg, 'strange-permission', fname, oct(perm))
 
