@@ -8,19 +8,20 @@
 #############################################################################
 
 import re
+import unicodedata
 try:
     from urlparse import urlparse
-except ImportError: # Python 3
+except ImportError:  # Python 3
     from urllib.parse import urlparse
 
 import rpm
 
 from Filter import addDetails, printError, printWarning
+from PostCheck import RPM_SCRIPTLETS
 from TagsCheck import VALID_GROUPS
 import AbstractCheck
 import Config
 import Pkg
-import Common
 
 # Don't check for hardcoded library paths in biarch packages
 DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
@@ -30,24 +31,23 @@ DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
 # be installed on biarch systems
 DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS = '/lib/(modules|cpp|perl5|rpm|hotplug|firmware)($|[\s/,])'
 
+
 def re_tag_compile(tag):
     if type(tag) == type([]):
         tag = '(?:' + '|'.join(tag) + ')'
-    r = "^%s\s*:\s*(\S+)\s*$" % tag
+    r = "^%s\s*:\s*(\S.*?)\s*$" % tag
     return re.compile(r, re.IGNORECASE)
 
-
-
 patch_regex = re_tag_compile('Patch(\d*)')
 applied_patch_regex = re.compile("^%patch(\d*)")
 applied_patch_p_regex = re.compile("\s-P\s+(\d+)\\b")
 applied_patch_pipe_regex = re.compile(r'\s%\{PATCH(\d+)\}\s*\|\s*(%\{?__)?patch\b')
 source_dir_regex = re.compile("^[^#]*(\$RPM_SOURCE_DIR|%{?_sourcedir}?)")
-obsolete_tags_regex = re_tag_compile(['Serial','Copyright'])
+obsolete_tags_regex = re_tag_compile(['Serial', 'Copyright'])
 buildroot_regex = re_tag_compile('BuildRoot')
 prefix_regex = re_tag_compile('Prefix')
 packager_regex = re_tag_compile('Packager')
-buildarch_regex = re_tag_compile(['BuildArch','BuildArchitectures'])
+buildarch_regex = re_tag_compile(['BuildArch', 'BuildArchitectures'])
 buildprereq_regex = re_tag_compile('BuildPreReq')
 prereq_regex = re_tag_compile('PreReq(\(.*\))')
 
@@ -68,8 +68,8 @@ defattr_regex = re.compile('^\s*%defattr\\b')
 attr_regex = re.compile('^\s*%attr\\b')
 section_regexs = dict(
     ([x, re.compile('^%' + x + '(?:\s|$)')]
-     for x in ['build', 'changelog', 'check', 'clean', 'description', 'files',
-               'install', 'package', 'prep'] + Common.RPM_SCRIPTLETS))
+     for x in ('build', 'changelog', 'check', 'clean', 'description', 'files',
+               'install', 'package', 'prep') + RPM_SCRIPTLETS))
 deprecated_grep_regex = re.compile(r'\b[ef]grep\b')
 
 # Only check for /lib, /usr/lib, /usr/X11R6/lib
@@ -81,10 +81,10 @@ hardcoded_library_path_regex = re.compile('^[^#]*((^|\s+|\.\./\.\.|\${?RPM_BUILD
 # https://bugzilla.redhat.com/118780 and bugs linked to that one.
 scriptlet_requires_regex = re.compile('^(PreReq|Requires)\([^\)]*,', re.IGNORECASE)
 
-DEFINE_RE='(^|\s)%(define|global)\s+'
+DEFINE_RE = '(^|\s)%(define|global)\s+'
 depscript_override_regex = re.compile(DEFINE_RE + '__find_(requires|provides)\s')
 depgen_disable_regex = re.compile(DEFINE_RE + '_use_internal_dependency_generator\s+0')
-patch_fuzz_override_regexp = re.compile(DEFINE_RE + '_default_patch_fuzz\s+(\d+)')
+patch_fuzz_override_regex = re.compile(DEFINE_RE + '_default_patch_fuzz\s+(\d+)')
 
 # See https://bugzilla.redhat.com/488146 for details
 indent_spaces_regex = re.compile('( \t|(^|\t)([^\t]{8})*[^\t]{4}[^\t]?([^\t][^\t.!?]|[^\t]?[.!?] )  )')
@@ -99,11 +99,16 @@ compop_regex = re.compile('[<>=]')
 setup_q_regex = re.compile(' -[A-Za-z]*q')
 setup_t_regex = re.compile(' -[A-Za-z]*T')
 setup_ab_regex = re.compile(' -[A-Za-z]*[ab]')
+autosetup_regex = re.compile('^\s*%autosetup(\s.*|$)')
+autosetup_n_regex = re.compile(' -[A-Za-z]*N')
+autopatch_regex = re.compile('^\s*%autopatch(?:\s|$)')
 
 filelist_regex = re.compile('\s+-f\s+\S+')
 pkgname_regex = re.compile('\s+(?:-n\s+)?(\S+)')
 tarball_regex = re.compile('\.(?:t(?:ar|[glx]z|bz2?)|zip)\\b', re.IGNORECASE)
 
+UNICODE_NBSP = unicodedata.lookup('NO-BREAK SPACE')
+
 
 def unversioned(deps):
     '''Yield unversioned dependency names from the given list.'''
@@ -111,12 +116,13 @@ def unversioned(deps):
         if not dep[1]:
             yield dep[0]
 
+
 def contains_buildroot(line):
     '''Check if the given line contains use of rpm buildroot.'''
     res = rpm_buildroot_regex.search(line)
     if res and \
-           (not res.group(1) or len(res.group(1)) % 2 == 0) and \
-           (not res.group(2) or len(res.group(2)) % 2 != 0):
+       (not res.group(1) or len(res.group(1)) % 2 == 0) and \
+       (not res.group(2) or len(res.group(2)) % 2 != 0):
         return True
     return False
 
@@ -127,10 +133,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         AbstractCheck.AbstractCheck.__init__(self, "SpecCheck")
         self._spec_file = None
 
-    def check(self, pkg):
-        if not pkg.isSource():
-            return
-
+    def check_source(self, pkg):
         wrong_spec = False
 
         # lookup spec file
@@ -159,6 +162,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         patches = {}
         applied_patches = []
         applied_patches_ifarch = []
+        patches_auto_applied = False
         source_dir = False
         buildroot = False
         configure_linenum = None
@@ -168,7 +172,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
         if_depth = 0
         ifarch_depth = -1
         current_section = 'package'
-        buildroot_clean = {'clean': False, 'install' : False}
+        buildroot_clean = {'clean': False, 'install': False}
         depscript_override = False
         depgen_disabled = False
         patch_fuzz_override = False
@@ -193,7 +197,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
 
         nbsp = chr(0xA0)
         if is_utf8:
-            nbsp = unichr(0xA0)
+            nbsp = UNICODE_NBSP
 
         for line in spec_lines:
 
@@ -240,7 +244,8 @@ class SpecCheck(AbstractCheck.AbstractCheck):
 
             if make_check_regex.search(line) and current_section not in \
                     ('check', 'changelog', 'package', 'description'):
-                printWarning(pkg, 'make-check-outside-check-section', line[:-1])
+                printWarning(pkg, 'make-check-outside-check-section',
+                             line[:-1])
 
             if current_section in buildroot_clean and \
                     not buildroot_clean[current_section] and \
@@ -264,6 +269,17 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         printWarning(pkg, 'setup-not-quiet')
                 if current_section != 'prep':
                     printWarning(pkg, 'setup-not-in-prep')
+            elif autopatch_regex.search(line):
+                patches_auto_applied = True
+                if current_section != 'prep':
+                    printWarning(pkg, '%autopatch-not-in-prep')
+            else:
+                res = autosetup_regex.search(line)
+                if res:
+                    if not autosetup_n_regex.search(res.group(1)):
+                        patches_auto_applied = True
+                    if current_section != 'prep':
+                        printWarning(pkg, '%autosetup-not-in-prep')
 
             if endif_regex.search(line):
                 if ifarch_depth == if_depth:
@@ -323,7 +339,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if current_section != 'changelog' and res and not \
                     (biarch_package_regex.match(pkg.name) or
                      hardcoded_lib_path_exceptions_regex.search(
-                            res.group(1).lstrip())):
+                         res.group(1).lstrip())):
                 printError(pkg, "hardcoded-library-path", "in",
                            res.group(1).lstrip())
 
@@ -356,7 +372,9 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                 res = buildarch_regex.search(line)
                 if res:
                     if res.group(1) != "noarch":
-                        printError(pkg, 'buildarch-instead-of-exclusivearch-tag', res.group(1))
+                        printError(pkg,
+                                   'buildarch-instead-of-exclusivearch-tag',
+                                   res.group(1))
                     else:
                         package_noarch[current_package] = True
 
@@ -386,25 +404,30 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                     reqs = Pkg.parse_deps(res.group(1))
                     for req in unversioned(reqs):
                         if compop_regex.search(req):
-                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                            printWarning(pkg,
+                                         'comparison-operator-in-deptoken',
                                          req)
 
                 res = provides_regex.search(line)
                 if res:
                     provs = Pkg.parse_deps(res.group(1))
                     for prov in unversioned(provs):
-                        printWarning(pkg, 'unversioned-explicit-provides', prov)
+                        printWarning(pkg, 'unversioned-explicit-provides',
+                                     prov)
                         if compop_regex.search(prov):
-                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                            printWarning(pkg,
+                                         'comparison-operator-in-deptoken',
                                          prov)
 
                 res = obsoletes_regex.search(line)
                 if res:
                     obses = Pkg.parse_deps(res.group(1))
                     for obs in unversioned(obses):
-                        printWarning(pkg, 'unversioned-explicit-obsoletes', obs)
+                        printWarning(pkg, 'unversioned-explicit-obsoletes',
+                                     obs)
                         if compop_regex.search(obs):
-                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                            printWarning(pkg,
+                                         'comparison-operator-in-deptoken',
                                          obs)
 
                 res = conflicts_regex.search(line)
@@ -412,7 +435,8 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                     confs = Pkg.parse_deps(res.group(1))
                     for conf in unversioned(confs):
                         if compop_regex.search(conf):
-                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                            printWarning(pkg,
+                                         'comparison-operator-in-deptoken',
                                          conf)
 
             if current_section == 'changelog':
@@ -429,7 +453,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         depgen_disable_regex.search(line) is not None
                 if not patch_fuzz_override:
                     patch_fuzz_override = \
-                        patch_fuzz_override_regexp.search(line) is not None
+                        patch_fuzz_override_regex.search(line) is not None
 
             if current_section == 'files':
 
@@ -468,7 +492,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             # If not checking spec file only, we're checking one inside a
             # SRPM -> skip this check to avoid duplicate warnings (#167)
             if spec_only and VALID_GROUPS and \
-                   line.lower().startswith("group:"):
+               line.lower().startswith("group:"):
                 group = line[6:].strip()
                 if group not in VALID_GROUPS:
                     printWarning(pkg, 'non-standard-group', group)
@@ -477,7 +501,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if hashPos != -1 and \
                     (hashPos == 0 or line[hashPos-1] in (" ", "\t")):
                 for match in AbstractCheck.macro_regex.findall(
-                    line[hashPos+1:]):
+                        line[hashPos+1:]):
                     res = re.match('%+', match)
                     if len(res.group(0)) % 2:
                         printWarning(pkg, 'macro-in-comment', match)
@@ -516,13 +540,14 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             pkg.current_linenum = None
 
         # process gathered info
-        for pnum, pfile in patches.items():
-            if pnum in applied_patches_ifarch:
-                printWarning(pkg, "%ifarch-applied-patch", "Patch%d:" % pnum,
-                             pfile)
-            if pnum not in applied_patches:
-                printWarning(pkg, "patch-not-applied", "Patch%d:" % pnum,
-                             pfile)
+        if not patches_auto_applied:
+            for pnum, pfile in patches.items():
+                if pnum in applied_patches_ifarch:
+                    printWarning(pkg, "%ifarch-applied-patch",
+                                 "Patch%d:" % pnum, pfile)
+                if pnum not in applied_patches:
+                    printWarning(pkg, "patch-not-applied",
+                                 "Patch%d:" % pnum, pfile)
 
         # Rest of the checks require a real spec file
         if not self._spec_file:
@@ -562,7 +587,7 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                 for src in sources:
                     (url, num, flags) = src
                     (scheme, netloc) = urlparse(url)[0:2]
-                    if flags & 1: # rpmspec.h, rpm.org ticket #123
+                    if flags & 1:  # rpmspec.h, rpm.org ticket #123
                         srctype = "Source"
                     else:
                         srctype = "Patch"
@@ -620,8 +645,8 @@ example using iconv(1).''',
 use a directory for building, use $RPM_BUILD_ROOT instead.''',
 
 'patch-not-applied',
-'''A patch is included in your package but was not applied. Refer to the patches
-documentation to see what's wrong.''',
+'''A patch is included in your package but was not applied. Refer to the
+patches documentation to see what's wrong.''',
 
 'obsolete-tag',
 '''The following tags are obsolete: Copyright and Serial. They must
@@ -800,8 +825,8 @@ is output by rpm and the message should contain more information.''',
 
 'comparison-operator-in-deptoken',
 '''This dependency token contains a comparison operator (<, > or =).  This is
-usually not intended and may be caused by missing whitespace between the token's
-name, the comparison operator and the version string.''',
+usually not intended and may be caused by missing whitespace between the
+token's name, the comparison operator and the version string.''',
 
 'macro-in-comment',
 '''There is a unescaped macro after a shell style comment in the specfile.
@@ -820,9 +845,10 @@ intended contents.''',
 
 'patch-fuzz-is-changed',
 '''The internal patch fuzz value was changed, and could hide patchs issues, or
-could lead to applying a patch at the wrong location. Usually, this is often the 
-sign that someone didn't check if a patch is still needed and do not want to rediff
-it. It is usually better to rediff the patch and try to send it upstream.'''
+could lead to applying a patch at the wrong location. Usually, this is often
+the sign that someone didn't check if a patch is still needed and do not want
+to rediff it. It is usually better to rediff the patch and try to send it
+upstream.'''
 )
 
 # SpecCheck.py ends here
