@@ -13,7 +13,7 @@ import re
 import time
 try:
     from urlparse import urlparse
-except ImportError: # Python 3
+except ImportError:  # Python 3
     from urllib.parse import urlparse
 
 import rpm
@@ -42,8 +42,8 @@ DEFAULT_VALID_LICENSES = (
     # the full name).  Updated 2010-02-01.
     'Academic Free License',
     'Adaptive Public License',
-    'AGPLv3', # Affero GNU Public License
-    'AGPLv3+', # Affero GNU Public License
+    'AGPLv3',  # Affero GNU Public License
+    'AGPLv3+',  # Affero GNU Public License
     'Apache License',
     'Apache Software License',
     'Apple Public Source License',
@@ -52,7 +52,7 @@ DEFAULT_VALID_LICENSES = (
     'BSD',
     'Boost Software License',
     'Computer Associates Trusted Open Source License',
-    'CDDL', # Common Development and Distribution License
+    'CDDL',  # Common Development and Distribution License
     'Common Public Attribution License',
     'CUA Office Public License',
     'EU DataGrid Software License',
@@ -83,7 +83,7 @@ DEFAULT_VALID_LICENSES = (
     'MirOS License',
     'MIT',
     'Motosoto License',
-    'MPL', # Mozilla Public License
+    'MPL',  # Mozilla Public License
     'Multics License',
     'NASA Open Source Agreement',
     'Naumen Public License',
@@ -92,13 +92,13 @@ DEFAULT_VALID_LICENSES = (
     'Non-profit Open Software License',
     'NTP License',
     'OCLC Research Public License',
-    'OFL', # Open Font License
+    'OFL',  # Open Font License
     'Open Group Test Suite License',
     'Open Software License',
     'PHP License',
-    'Python license', # CNRI Python License
+    'Python license',  # CNRI Python License
     'Python Software Foundation License',
-    'QPL', # Qt Public License
+    'QPL',  # Qt Public License
     'RealNetworks Public Source License',
     'Reciprocal Public License',
     'Ricoh Source Code Public License',
@@ -121,8 +121,8 @@ DEFAULT_VALID_LICENSES = (
     'Creative Commons Attribution-NonCommercial-ShareAlike',
     'Creative Commons Attribution-ShareAlike',
     # Others:
-    'Design Public License', # ???
-    'GFDL', # GNU Free Documentation License
+    'Design Public License',  # ???
+    'GFDL',  # GNU Free Documentation License
     'LaTeX Project Public License',
     'OpenContent License',
     'Open Publication License',
@@ -397,10 +397,11 @@ BAD_WORDS = {
     'xwindows': 'X'
     }
 
-DEFAULT_INVALID_REQUIRES = ('^is$', '^not$', '^owned$', '^by$', '^any$', '^package$', '^libsafe\.so\.')
+DEFAULT_INVALID_REQUIRES = ('^is$', '^not$', '^owned$', '^by$', '^any$',
+                            '^package$', '^libsafe\.so\.')
 
 VALID_GROUPS = Config.getOption('ValidGroups', None)
-if VALID_GROUPS is None: # get defaults from rpm package only if it's not set
+if VALID_GROUPS is None:  # get defaults from rpm package only if it's not set
     VALID_GROUPS = Pkg.get_default_valid_rpmgroups()
 VALID_LICENSES = Config.getOption('ValidLicenses', DEFAULT_VALID_LICENSES)
 INVALID_REQUIRES = map(re.compile, Config.getOption('InvalidRequires', DEFAULT_INVALID_REQUIRES))
@@ -440,6 +441,8 @@ for path in ('%perl_archlib', '%perl_vendorarch', '%perl_sitearch',
         private_so_paths.add(re.sub(r'/lib(?=/|$)', '/lib64', epath))
 
 _enchant_checkers = {}
+
+
 def spell_check(pkg, str, fmt, lang, ignored):
 
     dict_found = True
@@ -452,9 +455,9 @@ def spell_check(pkg, str, fmt, lang, ignored):
         if not checker and lang not in _enchant_checkers:
             try:
                 checker = enchant.checker.SpellChecker(
-                    lang, filters = [ enchant.tokenize.EmailFilter,
-                                      enchant.tokenize.URLFilter,
-                                      enchant.tokenize.WikiWordFilter ])
+                    lang, filters=[enchant.tokenize.EmailFilter,
+                                   enchant.tokenize.URLFilter,
+                                   enchant.tokenize.WikiWordFilter])
             except enchant.DictNotFoundError:
                 printInfo(pkg, 'enchant-dictionary-not-found', lang)
                 pass
@@ -464,6 +467,8 @@ def spell_check(pkg, str, fmt, lang, ignored):
             # squeeze whitespace to ease leading context check
             checker.set_text(re.sub(r'\s+', ' ', str))
             uppername = pkg.name.upper()
+            if use_utf8:
+                uppername = Pkg.to_utf8(uppername).decode('utf-8')
             upperparts = uppername.split('-')
             if lang.startswith('en'):
                 ups = [x + "'S" for x in upperparts]
@@ -533,6 +538,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
     def _unexpanded_macros(self, pkg, tagname, value, is_url=False):
         if not value:
             return
+        # str(value) because value might be a list
         for match in AbstractCheck.macro_regex.findall(str(value)):
             # Do not warn about %XX URL escapes
             if is_url and re.match('^%[0-9A-F][0-9A-F]$', match, re.I):
@@ -541,29 +547,31 @@ class TagsCheck(AbstractCheck.AbstractCheck):
 
     def check(self, pkg):
 
-        packager = pkg[rpm.RPMTAG_PACKAGER]
-        self._unexpanded_macros(pkg, 'Packager', packager)
-        if not packager:
+        packager = Pkg.b2s(pkg[rpm.RPMTAG_PACKAGER])
+        if packager:
+            self._unexpanded_macros(pkg, 'Packager', packager)
+            if Config.getOption('Packager') and \
+               not packager_regex.search(packager):
+                printWarning(pkg, 'invalid-packager', packager)
+        else:
             printError(pkg, 'no-packager-tag')
-        elif Config.getOption('Packager') and \
-                not packager_regex.search(packager):
-            printWarning(pkg, 'invalid-packager', packager)
 
         version = pkg[rpm.RPMTAG_VERSION]
-        self._unexpanded_macros(pkg, 'Version', version)
-        if not version:
-            printError(pkg, 'no-version-tag')
-        else:
+        if version:
+            self._unexpanded_macros(pkg, 'Version', version)
             res = invalid_version_regex.search(version)
             if res:
                 printError(pkg, 'invalid-version', version)
+        else:
+            printError(pkg, 'no-version-tag')
 
         release = pkg[rpm.RPMTAG_RELEASE]
-        self._unexpanded_macros(pkg, 'Release', release)
-        if not release:
+        if release:
+            self._unexpanded_macros(pkg, 'Release', release)
+            if release_ext and not extension_regex.search(release):
+                printWarning(pkg, 'not-standard-release-extension', release)
+        else:
             printError(pkg, 'no-release-tag')
-        elif release_ext and not extension_regex.search(release):
-            printWarning(pkg, 'not-standard-release-extension', release)
 
         epoch = pkg[rpm.RPMTAG_EPOCH]
         if epoch is None:
@@ -577,13 +585,13 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         if use_epoch:
             for o in (x for x in pkg.obsoletes() if x[1] and x[2][0] is None):
                 printWarning(pkg, 'no-epoch-in-obsoletes',
-                             apply(Pkg.formatRequire, o))
+                             Pkg.formatRequire(*o))
             for c in (x for x in pkg.conflicts() if x[1] and x[2][0] is None):
                 printWarning(pkg, 'no-epoch-in-conflicts',
-                             apply(Pkg.formatRequire, c))
+                             Pkg.formatRequire(*c))
             for p in (x for x in pkg.provides() if x[1] and x[2][0] is None):
                 printWarning(pkg, 'no-epoch-in-provides',
-                             apply(Pkg.formatRequire, p))
+                             Pkg.formatRequire(*p))
 
         name = pkg.name
         deps = pkg.requires() + pkg.prereq()
@@ -591,7 +599,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         is_devel = FilesCheck.devel_regex.search(name)
         is_source = pkg.isSource()
         for d in deps:
-            value = apply(Pkg.formatRequire, d)
+            value = Pkg.formatRequire(*d)
             if use_epoch and d[1] and d[2][0] is None and \
                     not d[0].startswith('rpmlib('):
                 printWarning(pkg, 'no-epoch-in-dependency', value)
@@ -675,25 +683,29 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         ignored_words.update((x[0] for x in pkg.conflicts()))
         ignored_words.update((x[0] for x in pkg.obsoletes()))
 
-        summary = pkg[rpm.RPMTAG_SUMMARY]
-        if not summary:
-            printError(pkg, 'no-summary-tag')
-        else:
-            if not pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+        langs = pkg[rpm.RPMTAG_HEADERI18NTABLE]
+        if langs:
+            langs = [Pkg.b2s(x) for x in langs]
+
+        summary = Pkg.b2s(pkg[rpm.RPMTAG_SUMMARY])
+        if summary:
+            if not langs:
                 self._unexpanded_macros(pkg, 'Summary', summary)
             else:
-                for lang in pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                for lang in langs:
                     self.check_summary(pkg, lang, ignored_words)
-
-        description = pkg[rpm.RPMTAG_DESCRIPTION]
-        if not description:
-            printError(pkg, 'no-description-tag')
         else:
-            if not pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+            printError(pkg, 'no-summary-tag')
+
+        description = Pkg.b2s(pkg[rpm.RPMTAG_DESCRIPTION])
+        if description:
+            if not langs:
                 self._unexpanded_macros(pkg, '%description', description)
             else:
-                for lang in pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                for lang in langs:
                     self.check_description(pkg, lang, ignored_words)
+        else:
+            printError(pkg, 'no-description-tag')
 
         group = pkg[rpm.RPMTAG_GROUP]
         self._unexpanded_macros(pkg, 'Group', group)
@@ -714,7 +726,8 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         if not changelog:
             printError(pkg, 'no-changelogname-tag')
         else:
-            clt = pkg[rpm.RPMTAG_CHANGELOGTEXT]
+            changelog = [Pkg.b2s(x) for x in changelog]
+            clt = [Pkg.b2s(x) for x in pkg[rpm.RPMTAG_CHANGELOGTEXT]]
             if use_version_in_changelog:
                 ret = changelog_version_regex.search(changelog[0])
                 if not ret and clt:
@@ -728,7 +741,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                     # only check when source name correspond to name
                     if srpm[0:-8] == '%s-%s-%s' % (name, version, release):
                         expected = [version + '-' + release]
-                        if epoch is not None: # regardless of use_epoch
+                        if epoch is not None:  # regardless of use_epoch
                             expected[0] = str(epoch) + ':' + expected[0]
                         # Allow EVR in changelog without release extension,
                         # the extension is often a macro or otherwise dynamic.
@@ -748,13 +761,13 @@ class TagsCheck(AbstractCheck.AbstractCheck):
 
             clt = pkg[rpm.RPMTAG_CHANGELOGTIME][0]
             if clt:
-                clt -= clt % (24*3600) # roll back to 00:00:00, see #246
+                clt -= clt % (24*3600)  # roll back to 00:00:00, see #246
                 if clt < oldest_changelog_timestamp:
                     printWarning(pkg, 'changelog-time-overflow',
                                  time.strftime("%Y-%m-%d", time.gmtime(clt)))
                 elif clt > time.time():
                     printError(pkg, 'changelog-time-in-future',
-                                 time.strftime("%Y-%m-%d", time.gmtime(clt)))
+                               time.strftime("%Y-%m-%d", time.gmtime(clt)))
 
 #         for provide_name in (x[0] for x in pkg.provides()):
 #             if name == provide_name:
@@ -783,13 +796,13 @@ class TagsCheck(AbstractCheck.AbstractCheck):
 
         for tag in ('URL', 'DistURL', 'BugURL'):
             if hasattr(rpm, 'RPMTAG_%s' % tag.upper()):
-                url = pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())]
-                self._unexpanded_macros(pkg, tag, url, is_url = True)
+                url = Pkg.b2s(pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())])
+                self._unexpanded_macros(pkg, tag, url, is_url=True)
                 if url:
                     (scheme, netloc) = urlparse(url)[0:2]
                     if not scheme or not netloc or "." not in netloc or \
                             scheme not in ('http', 'https', 'ftp') or \
-                            (Config.getOption('InvalidURL') and \
+                            (Config.getOption('InvalidURL') and
                              invalid_url_regex.search(url)):
                         printWarning(pkg, 'invalid-url', tag, url)
                     else:
@@ -803,7 +816,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         for o in (x for x in obs_names if x not in prov_names):
             printWarning(pkg, 'obsolete-not-provided', o)
         for o in pkg.obsoletes():
-            value = apply(Pkg.formatRequire, o)
+            value = Pkg.formatRequire(*o)
             self._unexpanded_macros(pkg, 'Obsoletes %s' % (value,), value)
 
         # TODO: should take versions, <, <=, =, >=, > into account here
@@ -816,11 +829,11 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             printError(pkg, 'useless-provides', p)
 
         for p in pkg.provides():
-            value = apply(Pkg.formatRequire, p)
+            value = Pkg.formatRequire(*p)
             self._unexpanded_macros(pkg, 'Provides %s' % (value,), value)
 
         for c in pkg.conflicts():
-            value = apply(Pkg.formatRequire, c)
+            value = Pkg.formatRequire(*c)
             self._unexpanded_macros(pkg, 'Conflicts %s' % (value,), value)
 
         obss = pkg.obsoletes()
@@ -829,9 +842,10 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             for prov in provs:
                 for obs in obss:
                     if Pkg.rangeCompare(obs, prov):
-                        printWarning(pkg, 'self-obsoletion', '%s obsoletes %s' %
-                                     (apply(Pkg.formatRequire, obs),
-                                      apply(Pkg.formatRequire, prov)))
+                        printWarning(pkg, 'self-obsoletion',
+                                     '%s obsoletes %s' %
+                                     (Pkg.formatRequire(*obs),
+                                      Pkg.formatRequire(*prov)))
 
         expfmt = rpm.expandMacro("%{_build_name_fmt}")
         if pkg.isSource():
@@ -846,8 +860,8 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         for tag in ('Distribution', 'DistTag', 'ExcludeArch', 'ExcludeOS',
                     'Vendor'):
             if hasattr(rpm, 'RPMTAG_%s' % tag.upper()):
-                self._unexpanded_macros(
-                    pkg, tag, pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())])
+                self._unexpanded_macros(pkg, tag,
+                    Pkg.b2s(pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())]))
 
         for path in private_so_paths:
             for fname, pkgfile in pkg.files().items():
@@ -855,8 +869,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                     for prov in pkgfile.provides:
                         if so_dep_regex.search(prov[0]):
                             printWarning(pkg, "private-shared-object-provides",
-                                         fname, apply(Pkg.formatRequire, prov))
-
+                                         fname, Pkg.formatRequire(*prov))
 
     def check_description(self, pkg, lang, ignored_words):
         description = pkg.langtag(rpm.RPMTAG_DESCRIPTION, lang)
@@ -1055,8 +1068,8 @@ leave out the Provides.''',
 the package was buggy.''',
 
 'no-epoch-tag',
-'''There is no Epoch tag in your package. You have to specify an epoch using the
-Epoch tag.''',
+'''There is no Epoch tag in your package. You have to specify an epoch using
+the Epoch tag.''',
 
 'unreasonable-epoch',
 '''The value of your Epoch tag is unreasonably large (> 99).''',
@@ -1083,8 +1096,8 @@ naming. This BuildRequires dependency will not be resolved on lib64 platforms
 (eg. amd64).''',
 
 'explicit-lib-dependency',
-'''You must let rpm find the library dependencies by itself. Do not put unneeded
-explicit Requires: tags.''',
+'''You must let rpm find the library dependencies by itself. Do not put
+unneeded explicit Requires: tags.''',
 
 'useless-provides',
 '''This package provides 2 times the same capacity. It should only provide it
